<?php

//////////////////////////////////////////////////////////////
//===========================================================
// WEBUZO
//===========================================================
// Inspired by the DESIRE to be the BEST OF ALL
// ----------------------------------------------------------
// Started by: Pulkit
// Date:       10th Jan 2009
// Time:       21:00 hrs
// Site:       https://webuzo.com/ (WEBUZO)
// ----------------------------------------------------------
// Please Read the Terms of Use at https://webuzo.com/terms
// ----------------------------------------------------------
//===========================================================
// (c) Softaculous Ltd.
//===========================================================
//////////////////////////////////////////////////////////////

// Last Updated : 18/05/2021
// Version : 1.0.0

error_reporting(0);

// Debug code
function webuzo_died(){
	print_r(error_get_last());
}

//register_shutdown_function('webuzo_died');

class webuzo extends Module {
	
	/**
	 * @var string The version of this module
	 */
	private static $version = "1.0.0";
	/**
	 * @var string The authors of this module
	 */
	private static $authors = array(array('name'=>"Softaculous Ltd.",'url'=>"https://webuzo.com"));
	
	/**
	 * Initializes the module
	 */
	function __construct(){
		
		$this->loglevel = 2;
		
		// Load components required by this module
		Loader::loadComponents($this, array("Input"));
		
		// Load the language required by this module
		Language::loadLang("webuzo", null, dirname(__FILE__) . DS . "language" . DS);
	}	
	
	/**
	 * Returns the name of this module
	 *
	 * @return string The common name of this module
	 */
	function getName(){
		return Language::_("webuzo.name", true);
	}
	
	/**
	 * Returns the version of this module
	 *
	 * @return string The current version of this module
	 */
	function getVersion() {
		return self::$version;
	}
	
	/**
	 * Returns the version of this module
	 *
	 * @return string The current version of this module
	 */
	function getDescription() {
		return Language::_("webuzo.desc", true);
	}

	/**
	 * Returns the name and URL for the authors of this module
	 *
	 * @return array A numerically indexed array that contains an array with key/value pairs for 'name' and 'url', representing the name and URL of the authors of this module
	 */
	function getAuthors() {
		return self::$authors;
	}
	
	/**
	 * Returns a noun used to refer to a module row (e.g. "Server", "VPS", "Reseller Account", etc.)
	 *
	 * @return string The noun used to refer to a module row
	 */
	function moduleRowName() {
		return Language::_("webuzo.module_row", true);
	}
	
	/**
	 * Returns the value used to identify a particular service
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @return string A value used to identify this service amongst other similar services
	 */
	function getServiceName($service) {
		foreach ($service->fields as $field) {
			if ($field->key == "webuzo_domain"){
				return $field->value;
			}
		}
		return null;
	}
	
	/**
	 * Returns the key used to identify the primary field from the set of module row meta fields.
	 * This value can be any of the module row meta fields.
	 *
	 * @return string The key used to identify the primary field from the set of module row meta fields
	 */
	function moduleRowMetaKey() {
		return "server_name";
	}
	
	/**
	 * Returns a noun used to refer to a module row in plural form (e.g. "Servers", "VPSs", "Reseller Accounts", etc.)
	 *
	 * @return string The noun used to refer to a module row in plural form
	 */
	function moduleRowNamePlural() {
		return Language::_("webuzo.module_row_plural", true);
	}
	
	/**
	 * Returns a noun used to refer to a module group (e.g. "Server Group", "Cloud", etc.)
	 *
	 * @return string The noun used to refer to a module group
	 */
	function moduleGroupName() {
		return Language::_("webuzo.module_group", true);
	}
	
	/**
	 * Returns an array of available service delegation order methods. The module
	 * will determine how each method is defined. For example, the method "first"
	 * may be implemented such that it returns the module row with the least number
	 * of services assigned to it.
	 *
	 * @return array An array of order methods in key/value pairs where the key is the type to be stored for the group and value is the name for that option
	 * @see Module::selectModuleRow()
	 */
	function getGroupOrderOptions() {
		return array('first'=>Language::_("webuzo.order_options.first", true));
	}
	
	
	/**
	 * Determines which module row should be attempted when a service is provisioned
	 * for the given group based upon the order method set for that group.
	 *
	 * @return int The module row ID to attempt to add the service with
	 * @see Module::getGroupOrderOptions()
	 */
	function selectModuleRow($module_group_id) {
		if (!isset($this->ModuleManager)){
			Loader::loadModels($this, array("ModuleManager"));
		}
		
		$group = $this->ModuleManager->getGroup($module_group_id);
		
		if ($group) {
			switch ($group->add_order) {
				default:
				case "first":
					
					foreach ($group->rows as $row) {
						return $row->id;
					}
					
					break;
			}
		}
		return 0;
	}
	
	/**
	 * Returns the rendered view of the manage module page
	 *
	 * @param mixed $module A stdClass object representing the module and its rows
	 * @param array $vars An array of post data submitted to or on the manage module page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the manager module page
	 */
	function manageModule($module, array &$vars) {
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("manage", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));
		
		$this->view->set("module", $module);
		
		return $this->view->fetch();
	}
	
	/**
	 * Returns the rendered view of the add module row page
	 *
	 * @param array $vars An array of post data submitted to or on the add module row page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the add module row page
	 */
	function manageAddRow(array &$vars) {
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("add_row", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));
		
		$this->view->set("vars", (object)$vars);
		return $this->view->fetch();	
	}
	
	/**
	 * Returns the rendered view of the edit module row page
	 *
	 * @param stdClass $module_row The stdClass representation of the existing module row
	 * @param array $vars An array of post data submitted to or on the edit module row page (used to repopulate fields after an error)
	 * @return string HTML content containing information to display when viewing the edit module row page
	 */	
	function manageEditRow($module_row, array &$vars) {
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("edit_row", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html", "Widget"));
		
		if (empty($vars))
			$vars = $module_row->meta;
		
		$this->view->set("vars", (object)$vars);
		return $this->view->fetch();
	}
	
	/**
	 * Retrieves the module row given the server or server group
	 *
	 * @param string $module_row The module row ID
	 * @param string $module_group The module group (optional, default "")
	 * @return mixed An stdClass object representing the module row, or null if it could not be determined
	 */
	private function getModuleRowByServer($module_row, $module_group = "") {
		
		// Fetch the module row available for this package
		$row = null;
		if ($module_group == "") {
			if ($module_row > 0) {
				$row = $this->getModuleRow($module_row);
			}
			else {
				$rows = $this->getModuleRows();
				
				if (isset($rows[0]))
					$row = $rows[0];
				unset($rows);
			}
		}
		else {
			// Fetch the 1st server from the list of servers in the selected group
			$rows = $this->getModuleRows($module_group);
			
			if (isset($rows[0]))
				$row = $rows[0];
			unset($rows);
		}
		return $row;
	}	
	
	/**
	 * Returns an array of key values for fields stored for a module, package,
	 * and service under this module, used to substitute those keys with their
	 * actual module, package, or service meta values in related emails.
	 *
	 * @return array A multi-dimensional array of key/value pairs where each key is one of 'module', 'package', or 'service' and each value is a numerically indexed array of key values that match meta fields under that category.
	 * @see Modules::addModuleRow()
	 * @see Modules::editModuleRow()
	 * @see Modules::addPackage()
	 * @see Modules::editPackage()
	 * @see Modules::addService()
	 * @see Modules::editService()
	 */
	function getEmailTags() {
		return array(
			'module' => array("host", "port"),
			'package' => array(),
			'service' => array("uuid", "webuzo_domain", "webuzo_username", "webuzo_password", "webuzo_ip", "webuzo_additional_ips"
			)
		);
	}
	
	/**
	 * Returns all fields to display to an admin attempting to add a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */
	function getAdminAddFields($package, $vars=null) {
		
		Loader::loadHelpers($this, array("Html"));
		$fields = new ModuleFields();
		
		// Create domain label
		$domain = $fields->label("Domain", "webuzo_domain");
		// Create domain field and attach to domain label
		$domain->attach($fields->fieldText("webuzo_domain", $this->Html->ifSet($vars->webuzo_domain), array('id'=>"webuzo_domain")));
		// Set the label as a field
		$fields->setField($domain);
		unset($domain);
		
		// Create username label
		$username = $fields->label("Username", "webuzo_username");
		// Create username field and attach to username label
		$username->attach($fields->fieldText("webuzo_username", $this->Html->ifSet($vars->webuzo_username), array('id'=>"webuzo_username")));
		// Set the label as a field
		$fields->setField($username);
		unset($username);
		
		// Create password label
		$password = $fields->label("Password", "webuzo_password");
		// Create password field and attach to password label
		$password->attach($fields->fieldText("webuzo_password", $this->Html->ifSet($vars->webuzo_password), array('id'=>"webuzo_password")));
		// Set the label as a field
		$fields->setField($password);
		unset($password);
		
		return $fields;
	}
	
	/**
	 * Returns all fields used when adding/editing a package, including any
	 * javascript to execute when the page is rendered with these fields.
	 *
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */
	 
	function getPackageFields($vars=null) {
		
		Loader::loadHelpers($this, array("Form", "Html"));
		$fields = new ModuleFields();
		
		// Show nodes, and set javascript field toggles
		$this->Form->setOutput(true);
		
		
		// Fetch all packages available for the given server or server group
		$module_row = $this->getModuleRowByServer((isset($vars->module_group) ? $vars->module_group : 0), (isset($vars->module_group) ? $vars->module_group : ""));
		
		$templates = array();
		$nodes = array();
		$tmp_servers = array();
		$plans = array();
		$result = new Record();
		$data = $result->select(array("module_row_id","key","value","encrypted"))->from("module_row_meta")->fetchAll();
	
		// Build the server list
		foreach($data as $k => $v){
			
			$val = $v->value;
			
			// Key and pass are stored encrypted so we have to decrypt it
			if($v->key == 'apiuser' || $v->key == 'apikey'){
				
				// If it is encrypted then only Decrypt it
				if(!empty($v->encrypted)){
					$val = $this->getdecryptpass($v->value);
				}
				
				if(empty($val)){
					$val = $v->value;
				}
			}			
			$_servers[$v->module_row_id][$v->key] = $val;
		}
		
		$tmp_servers = $_servers;
		
		if($this->loglevel > 1){
			$this->wlog('[webuzo::getPackageFields] Servers :'.var_export($_servers, true));
		}

		//Just to be on the safe side $tmp_servers is used instead of $_servers
		$_module_row = (!empty($vars->module_row) ? $vars->module_row : array_shift(array_keys($tmp_servers)));
		
		$ip = $_servers[$_module_row]['host'];
		$apiuser = $_servers[$_module_row]['apiuser'];
		$pass = $_servers[$_module_row]['apikey'];
		
		//Make an api call
		$data = $this->make_api_call($ip, $apiuser, $pass, 'index.php?act=add_user');
		
		//Configure plans
		foreach($data['plans'] as $k => $v){
			$plans[$k] = $v['plan_name'].' - '.$k;
		}
		
		/*// Build the default node / group field
		$tmp_node_grp['Auto Select Server'] = 'Auto Select Server';
		
		foreach ($data['servergroups'] as $k => $v){
			
			$tmp_node_grp[$k.' - [G] '.$v['sg_name']] = $k.' - [G] '.$v['sg_name'];
			
			foreach ($data['servers'] as $m => $n){
				if($n['sgid'] == $k){
					$tmp_node_grp[$m." - ".$n['server_name']] = "\t".$m." - ".$n['server_name'];
				}
			}
		}*/
		
		// We are taking this from URL as on change of anything it is posting the fields
		// http://DOMAIN/DIR/admin/packages/add/
		$_tmp_url = basename($GLOBALS['_ENV']['HTTP_REFERER']);
		
		$fields->setHtml("
		<script type=\"text/javascript\">
			$(document).ready(function() {
				toggle_webuzo_fields();
			});
			
			$('#webuzo_plan').change(function() {
				fetchModuleOptions();
			});

			function toggle_webuzo_fields() {
				// Hide fields dependent on this value
				if ($('#kernel').val() == '') {
					$('#webuzo_plan').parent('li').hide();
					$('#server_groups').parent('li').hide();
				}
				// Show fields dependent on this value
				else{
					$('#webuzo_plan').parent('li').show();
					$('#server_groups').parent('li').show();
				}
			}
		</script>");
			
		//SHOW ONLY SERVER, PLANS, TYPE AND SERVERGROUPS
		
		// Selecting Webuzo Plans
		$plan = $fields->label("Select Plan", "webuzo_plan");
		$plan->attach($fields->fieldSelect("meta[plan]", $plans,
		$this->Html->ifSet($vars->meta['plan']), array('id' => "webuzo_plan")));					
		$fields->setField($plan);
		
		/*// Selecting the Server Groups
		$server_groups = $fields->label("Server Groups", "server_groups");
		$server_groups->attach($fields->fieldSelect("meta[sg]", $tmp_node_grp,
		$this->Html->ifSet($vars->meta['sg']), array('id' => "server_groups")));					
		$fields->setField($server_groups);*/
		
		$this->log($module_row->meta->host . "|List Plans", serialize($plan), "input", true);

		return $fields;
	}
	
	/**
	 * Returns all fields to display to a client attempting to add a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */	
	function getClientAddFields($package, $vars=null) {
		Loader::loadHelpers($this, array("Html"));
		
		// Fetch all packages available for the given server or server group
		$module_row = $this->getModuleRowByServer((isset($vars->module_group) ? $vars->module_group : 0), (isset($vars->module_group) ? $vars->module_group : ""));
		
		$fields = new ModuleFields();
		
		// Create domain label
		$domain = $fields->label("Domain", "webuzo_domain");
		// Create domain field and attach to domain label
		$domain->attach($fields->fieldText("webuzo_domain", $this->Html->ifSet($vars->webuzo_domain), array('id'=>"webuzo_domain")));
		// Set the label as a field
		$fields->setField($domain);
		unset($domain);
		
		// Create username label
		$username = $fields->label("Username", "webuzo_username");
		// Create username field and attach to username label
		$username->attach($fields->fieldText("webuzo_username", $this->Html->ifSet($vars->webuzo_username), array('id'=>"webuzo_username")));
		// Set the label as a field
		$fields->setField($username);
		unset($username);
		
		// Create password label
		$password = $fields->label("Password", "webuzo_password");
		// Create password field and attach to password label
		$password->attach($fields->fieldPassword("webuzo_password", array('id'=>"webuzo_password")));
		// Set the label as a field
		$fields->setField($password);
		
		unset($password);
		
		// Confirm password label
		$confirm_password = $fields->label("Confirm Password", "webuzo_confirm_password");
		// Create confirm password field and attach to password label
		$confirm_password->attach($fields->fieldPassword("webuzo_confirm_password", array('id'=>"webuzo_confirm_password")));
		// Set the label as a field
		$fields->setField($confirm_password);
		unset($confirm_password);
		
		return $fields;
	}	
	
	/**
	 * Returns all fields to display to an admin attempting to edit a service with the module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param $vars stdClass A stdClass object representing a set of post fields
	 * @return ModuleFields A ModuleFields object, containg the fields to render as well as any additional HTML markup to include
	 */	
	function getAdminEditFields($package, $vars=null) {
		Loader::loadHelpers($this, array("Html"));
		
		$fields = new ModuleFields();
		
		// Create domain label
		$domain = $fields->label("Domain", "webuzo_domain");
		// Create domain field and attach to domain label
		$domain->attach($fields->fieldText("webuzo_domain", $this->Html->ifSet($vars->webuzo_domain), array('id'=>"webuzo_domain")));
		// Set the label as a field
		$fields->setField($domain);

		// Create username label
		$username = $fields->label("Username", "webuzo_username");
		// Create username field and attach to username label
		$username->attach($fields->fieldText("webuzo_username", $this->Html->ifSet($vars->webuzo_username), array('id'=>"webuzo_username")));
		// Set the label as a field
		$fields->setField($username);
		
		// Create password label
		$password = $fields->label("Password", "webuzo_password");
		// Create password field and attach to password label
		$password->attach($fields->fieldText("webuzo_password", $this->Html->ifSet($vars->webuzo_password), array('id'=>"webuzo_password")));
		// Set the label as a field
		$fields->setField($password);
		
		// uuid
		$uuid = $fields->label("uuid", "uuid");
		$uuid->attach($fields->fieldText("uuid", $this->Html->ifSet($vars->uuid), array('id' => "uuid")));
		$fields->setField($uuid);
		
		return $fields;
	}	
	
	/**
	 * Fetches the HTML content to display when viewing the service info in the
	 * admin interface.
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param stdClass $package A stdClass object representing the service's package
	 * @return string HTML content containing information to display when viewing the service info
	 */
	function getAdminServiceInfo($service, $package) {
		$row = $this->getModuleRow();
		// Get the service fields
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("admin_service_info", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$this->view->set("module_row", $row);
		$this->view->set("package", $package);
		$this->view->set("service", $service);
		$this->view->set("service_fields", $this->serviceFieldsToObject($service->fields));
		$this->view->set("view", $this->view->view);
		
		return 'Domain - '.$service_fields->webuzo_domain.' Username - '.$service_fields->webuzo_username;
		return $this->view->fetch();
	}
	
	/**
	 * Fetches the HTML content to display when viewing the service info in the
	 * client interface.
	 *
	 * @param stdClass $service A stdClass object representing the service
	 * @param stdClass $package A stdClass object representing the service's package
	 * @return string HTML content containing information to display when viewing the service info
	 */
	function getClientServiceInfo($service, $package) {
		$row = $this->getModuleRow();
		// Get the service fields
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		// Load the view into this object, so helpers can be automatically added to the view
		$this->view = new View("client_service_info", "default");
		$this->view->base_uri = $this->base_uri;
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));

		$this->view->set("module_row", $row);
		$this->view->set("package", $package);
		$this->view->set("service", $service);
		$this->view->set("service_fields", $this->serviceFieldsToObject($service->fields));
		$this->view->set("view", $this->view->view);
		
		return 'Hostname - '.$service_fields->webuzo_domain.' Username - '.$service_fields->webuzo_username;
		
		return $this->view->fetch();
	}
	
	/**
	 * Returns all tabs to display to an admin when managing a service whose
	 * package uses this module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @return array An array of tabs in the format of method => title. Example: array('methodName' => "Title", 'methodName2' => "Title2")
	 */
	function getAdminTabs($package) {
		return array(
			'tabActions' => "Control Panel",
		);
	}
	
	function webuzo_UI($package, $service, $url, $static_url, array $get=null, array $post=null) {
		
		global $virt_verify, $virt_errors;
		$service_fields = $this->serviceFieldsToObject($service->fields);
		$module_row = $this->getModuleRow($package->module_row);
		
		$params['package_id'] = $package->pricing[0]->package_id;
		$params['client_id'] = $service->client_id;
		$params['uuid'] = $service_fields->uuid;
		$params['username'] = $service_fields->webuzo_username;
		$params['service_id'] = $service->id;
	
		if($this->loglevel > 0){
			$this->wlog('[webuzo_UI] Service Fields : '.var_export($service_fields, 1));
		}
		
		if(empty($params['uuid'])){ //Checks for UUID
			$string = array(
				'value'=>'Account is not provisioned'
			);
			
			return implode("\n*", array_values($string));
		}
		
		// HTML file
		if(!empty($_REQUEST['wapi_mode'])){
			
			$ip = $module_row->meta->host;
			$apiuser = $module_row->meta->apiuser;
			$pass = $module_row->meta->apikey;
			$_POST['billing_ind'] = base64_encode($url);
			$_POST['billing_static'] = base64_encode($static_url);
			$path = 'index.php?'.http_build_query($_GET).'&';
			
			$res = $this->enduser_call($ip, $apiuser, $pass, $params['username'], $path, $_POST);		
			echo $res;
			
			die();
			exit(0);
			
		}
		
		// Static file
		if(!empty($_REQUEST['webuzo_static'])){
			
			$ip = $module_row->meta->host;
			
			$this->enduser_static_call($ip, $_REQUEST['webuzo_static']);
			
			die();
			exit(0);
			
		}
		
		return;
	}
	
	/**
	 * Actions tab (boot, reboot, shutdown, etc.)
	 *
	 * @param stdClass $package A stdClass object representing the current package
	 * @param stdClass $service A stdClass object representing the current service
	 * @param array $get Any GET parameters
	 * @param array $post Any POST parameters
	 * @param array $files Any FILES parameters
	 * @return string The string representing the contents of this tab
	 */
	function tabActions($package, $service, array $get=null, array $post=null, array $files=null) {
		
		$params['client_id'] = $service->client_id;
		$params['service_id'] = $service->id;
		
		$url = (!empty($_SERVER['HTTPS']) ? 'https://' : 'http://').$_SERVER['HTTP_HOST'].$this->base_uri.'clients/servicetab/'.$params['client_id'].'/'.$params['service_id'].'/tabActions?';
		
		// Webuzo Panel will be displayed
		echo $this->webuzo_UI($package, $service, $url.'wapi_mode=1&', $url.'webuzo_static=', $get, $post);
		
		$this->view = new View("tab_actions", "default");
		$this->view->base_uri = $this->base_uri;
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));	
		
		$this->view->set("client_id", $service->client_id);
		$this->view->set("service_id", $service->id);
		
		$this->view->set("view", $this->view->view);
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		return $this->view->fetch();
		
	}
	
	/**
	 * Returns all tabs to display to a client when managing a service whose
	 * package uses this module
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @return array An array of tabs in the format of method => title. Example: array('methodName' => "Title", 'methodName2' => "Title2")
	 */
	function getClientTabs($package) {
		return array(
			'tabClientActions' => "Control Panel",
		);
	}
	
	// Client Panel service Tabs
	function tabClientActions($package, $service, array $get=null, array $post=null, array $files=null) {
	
		$params['package_id'] = $package->pricing[0]->package_id;
		$params['service_id'] = $service->id;
		
		$modules_url = 'components/modules';
		
		$url = (!empty($_SERVER['HTTPS']) ? 'https://' : 'http://').$_SERVER['HTTP_HOST'].$this->base_uri.'services/manage/'.$params['service_id'].'/tabClientActions?';
				
		echo $this->webuzo_UI($package, $service, $url.'wapi_mode=1&', $url.'webuzo_static=', $get, $post);
		
		$this->view = new View("tab_client_actions", "default");
		$this->view->base_uri = $this->base_uri;
		
		// Load the helpers required for this view
		Loader::loadHelpers($this, array("Form", "Html"));
		
		$this->view->set("client_id", $service->client_id);
		$this->view->set("service_id", $service->id);
		
		$this->view->set("view", $this->view->view);
		$this->view->setDefaultView("components" . DS . "modules" . DS . "webuzo" . DS);
		
		return $this->view->fetch();
	}	
	
	/**
	 * Returns an array of service field to set for the service using the given input
	 *
	 * @param array $vars An array of key/value input pairs
	 * @param stdClass $package A stdClass object representing the package for the service
	 * @return array An array of key/value pairs representing service fields
	 */
	private function getFieldsFromInput(array $vars, $package) {		
		
		$fields = array(
			'domain' => isset($vars['webuzo_domain']) ? $vars['webuzo_domain'] : null,
			'username' => isset($vars['webuzo_username']) ? $vars['webuzo_username']: null,
			'password' => isset($vars['webuzo_password']) ? $vars['webuzo_password'] : null,
			'confirm_password' => isset($vars['webuzo_confirm_password']) ? $vars['webuzo_confirm_password'] : null,
		);
		
		
		
		return $fields;
	}
	
	
	/**
	 * Adds the service to the remote server. Sets Input errors on failure,
	 * preventing the service from being added.
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param array $vars An array of user supplied info to satisfy the request
	 * @param stdClass $parent_package A stdClass object representing the parent service's selected package (if the current service is an addon service)
	 * @param stdClass $parent_service A stdClass object representing the parent service of the service being added (if the current service is an addon service service and parent service has already been provisioned)
	 * @param string $status The status of the service being added. These include:
	 * 	- active
	 * 	- canceled
	 * 	- pending
	 * 	- suspended
	 * @return array A numerically indexed array of meta fields to be stored for this service containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 * @see Module::getModule()
	 * @see Module::getModuleRow()
	 */
	function addService($package, array $vars=null, $parent_package=null, $parent_service=null, $status="pending"){
	
		// This fetches the Server according to package selected.
		$row = $this->getModuleRow();
		
		$params = $this->getFieldsFromInput((array)$vars, $package);
	
		if($this->loglevel > 1){
			$this->wlog('[Webuzo:addService] Params : '.var_export($params, 1));
		}
	
		if($this->loglevel > 1){
			$this->wlog('[Webuzo:addService] Package Meta : '.var_export($package->meta, 1));
		}
		
		// Get the client details 
		Loader::loadModels($this, array("Clients"));
		$clients = $this->Clients->get($vars['client_id'], false);
		
		$this->validateService($package, $vars);

		if ($this->Input->errors()){
			return;
		}
		
		// Only provision the service if 'use_module' is true
		if ($vars['use_module'] == "true") {
			
			// Get the info from the server
			$pass = $row->meta->apikey;
			$apiuser = $row->meta->apiuser;
			$ip = $row->meta->host;
			$path = 'index.php?act=add_user';
				
			$server_group = '';
			$slave_server = '';
			
			// If user has configurable option for server group we give the priority to that so we will override the product value with configoption
			if(!empty($vars['configoptions']['server_group'])){
				$package->meta->sg = $vars['configoptions']['server_group'].' - [G]';
			}
			
			// Is it a Server group ?
			if(preg_match('/\[G\]/s', $package->meta->sg)){				
				$tmp_sg = array();
				$tmp_sg = explode('- [', $package->meta->sg);
				$server_group = trim($tmp_sg[0]);
			}
			
			// If we do not get server group we will search it for slave server
			if($server_group == ''){
				// Is user wants auto selection from server?
				if($package->meta->sg == 'Auto Select Server'){
					
					$slave_server = 'auto';
					
				// Or is it a particular Slave server ?
				}else{
					
					$tmp_ss = array();
					$tmp_ss = explode("-", (string)$package->meta->sg);
					$slave_server = trim($tmp_ss[0]);
				}
			}
			
			$post['server_group'] = $server_group;
			$post['slave_server'] = $slave_server;
			
			// Pass the user details 
			$post['plan'] = $package->meta->plan;
			$post['domain'] = $params["domain"];
			$post['user'] = $params["username"];
			$post['user_passwd'] = $params["password"];
			$post['cnf_user_passwd'] = $params['password'];
			$post['email'] = $clients->email;
	
			$post['billing_prefill'] = 1;
			$post['node_select'] = 1;
			$post['create_user'] = 1;
			
			$masked_array = $post;
			$masked_array['user_passwd'] = '***';
			
			$ret = array();
			
			//Log the creation of service
			$this->log($row->meta->host . "|create service", serialize($masked_array), "input", true);

			// Are there any configurable options
			if(!empty($vars['configoptions'])){
				foreach($vars['configoptions'] as $k => $v){
					if(!isset($post[$k])){
						$post[$k] = $v;
					}
				}
			}
			
			// API call
			$ret = $this->make_api_call($ip, $apiuser, $pass, $path, $post);
			
			$this->log($row->meta->host, $ret, "output", true);
			
			if(empty($ret['done'])){
				$create_error = implode('<br>', $ret['error']);
				$this->Input->setErrors(array('api' => array('internal' => 'Create Error :'.$create_error)));
				return;
			}
	
			// Was the account created
			if(empty($ret['done']['user'])){
				$create_error = implode('<br>', $ret['error']);
				$this->Input->setErrors(array('api' => array('internal' => 'Create Error :'.$create_error)));
			}
		
			// Fill the variables like ips, ips6, etc..
			if(!empty($ret['done']['user']['ips'])){
				$_ips = $ret['done']['user']['ips'];
			}
			
			if(!empty($ret['done']['user']['ipv6'])){
				$_ips6 = $ret['done']['user']['ipv6'];
			}
			
			if(!empty($ret['done']['user']['ipv6_subnet'])){
				$_ips6_subnet = $ret['done']['user']['ipv6_subnet'];
			}
			
		}// End of if(use_module)
		
		$tmp_ips = empty($_ips) ? array() : $_ips;
		
		if(!empty($_ips6_subnet)){
			foreach($_ips6_subnet as $k => $v){
				$tmp_ips[] = $v;
			}
		}
		
		if(!empty($_ips6)){
			foreach($_ips6 as $k => $v){
				$tmp_ips[] = $v;
			}
		}
		
		if(!empty($tmp_ips[0])){
			
			$primary_ip = $tmp_ips[0];
			
			// Extra IPs
			unset($tmp_ips[0]);
		}
		
		// Return service fields
		return array(
			array(
				'key' => "webuzo_domain",
				'value' => $vars['webuzo_domain'],
				'encrypted' => 0
			),
			array(
				'key' => "webuzo_username",
				'value' => $vars['webuzo_username'],
				'encrypted' => 0
			),
			array(
				'key' => "webuzo_password",
				'value' => $vars['webuzo_password'],
				'encrypted' => 1
			),
			array(
				'key' => "webuzo_ip",
				'value' => @$primary_ip,
				'encrypted' => 0
			),
			array(
				'key' => "webuzo_additional_ips",
				'value' => (!empty($tmp_ips) ? implode(', ', $tmp_ips) : null),
				'encrypted' => 0
			),
			array(
				'key' => "uuid",
				'value' => (isset($ret['done']['user']['uuid']) ? $ret['done']['user']['uuid'] : null),
				'encrypted' => 0
			)
		);
	}
	
	// Edit the Service
	function editService($package, $service, array $vars= null, $parent_package=null, $parent_service=null) {
		
		$module_row = $this->getModuleRow($package->module_row);
		$encrypted_fields = array("apiuser", "apikey");
		
		// Get the service fields
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		$params = $this->getFieldsFromInput((array)$vars, $package);
				
		// If the service was created wihtout using the module, we wont get vpsid.
		// If user wants to add it afterward we will get the vpsid from $_POST
		if(empty($service_fields->uuid) && !empty($_POST['uuid'])){
			$service_fields->uuid = $_POST['uuid'];
		}
		
		// Get the client details 
		Loader::loadModels($this, array("Clients"));
		$clients = $this->Clients->get((array)$service->client_id, false);
		
		$pass = $module_row->meta->apikey;
		$apiuser = $module_row->meta->apiuser;
		$ip = $module_row->meta->host;
		$path = 'index.php?act=add_user&user_name='.$params['username'];
		
		$data = $this->make_api_call($ip, $apiuser, $pass, $path);
		
		if(empty($data)){
			return 'Could not load the server data.'.$module_row->meta->host;
		}
	
		$post = $data['user']['p'];
		
		// Fetch the current service plan
		$plan = '';
		
		if (isset($package->meta->plan)) {
			$plan = $package->meta->plan;
		}
		
		if(!empty($service_fields->uuid)){
			$service_fields->uuid = $vars['uuid'];
		}
		
		$this->validateService($package, $vars);
	
		// Pass the user details 
		$post['plan'] = $plan;
		$post['domain'] = $params['domain'];
		$post['user'] = $params['username'];
		$post['email'] = $clients->email;
	
		if($vars['use_module'] == "true") {

			// Are there any configurable options
			if(!empty($vars['configoptions'])){
				foreach($vars['configoptions'] as $k => $v){
					if(!isset($post[$k])){
						$post[$k] = $v;
					}
				}
			}
			
			$post['billing_prefill'] = 1;
			$post['node_select'] = 1;
			$post['edit_user'] = 1;
			
			// Virtualizor API call
			$ret = $this->make_api_call($ip, $apiuser, $pass, $path, $post);
			
			if(empty($ret['done'])){
				$edit_error = implode('<br>', $ret['error']);
				$this->Input->setErrors(array('api' => array('internal' => 'Edit Error :'.$edit_error)));
				return;
			}
		}
	
		$fields = array();
	
		// Return all the service fields
		foreach ($service_fields as $key => $value){
			$fields[] = array('key' => $key,
							'value' => $value,
							'encrypted' => (in_array($key, $encrypted_fields) ? 1 : 0));
		}

		return $fields;
	}
	
	// Cancel the Service
	function cancelService($package, $service, $parent_package=null, $parent_service=null) {
		
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		// Will get the server key and pass
		$module_row = $this->getModuleRow($package->module_row);
		
		$pass = $module_row->meta->apikey;
		$apiuser = $module_row->meta->apiuser;
		$ip = $module_row->meta->host;
		$path = 'index.php?act=users&delete_user='.$service_fields->webuzo_username;
		
		$params = array('username' => $service_fields->webuzo_username);
		
		$this->log($row->meta->host . "|terminate User", serialize($params), "input", true);
		
		$ret = $this->make_api_call($ip, $apiuser, $pass, $path);
		
		$this->log($module_row->meta->host . "|User Terminated", $ret, "output", true);
		
		if(empty($ret)){
			$this->Input->setErrors(array('api' => array('internal' => 'Unable to cancel the service')));
		}
		
		return null;
	}
	
	// Cancel the Service
	function suspendService($package, $service, $parent_package=null, $parent_service=null) {
		
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		// Will get the server key and pass
		$module_row = $this->getModuleRow($package->module_row);
		
		$pass = $module_row->meta->apikey;
		$apiuser = $module_row->meta->apiuser;
		$ip = $module_row->meta->host;
		$path = 'index.php?act=users&suspend='.$service_fields->webuzo_username;
		
		$params = array('username' => $service_fields->webuzo_username);
		
		$this->log($row->meta->host . "|suspend webuzo user", serialize($params), "input", true);
		
		$ret = $this->make_api_call($ip, $apiuser, $pass, $path);
		
		$this->log($row->meta->host . "|User Suspended", $virt_resp, "output", true);
		
		if(empty($ret)){
			$this->Input->setErrors(array('api' => array('internal' => 'Unable to suspend the service')));
		}
		
		return null;
	}
	
	// Cancel the Service
	function unsuspendService($package, $service, $parent_package=null, $parent_service=null) {
		
		$service_fields = $this->serviceFieldsToObject($service->fields);
		
		// Will get the server key and pass
		$module_row = $this->getModuleRow($package->module_row);
		
		$pass = $module_row->meta->apikey;
		$apiuser = $module_row->meta->apiuser;
		$ip = $module_row->meta->host;
		$path = 'index.php?act=users&unsuspend='.$service_fields->webuzo_username;
		
		$params = array('username' => $service_fields->webuzo_username);
		
		$this->log($row->meta->host . "|Unsuspend webuzo user", serialize($params), "input", true);
		
		$ret = $this->make_api_call($ip, $apiuser, $pass, $path);
		
		$this->log($row->meta->host . "|User Unsuspended", $virt_resp, "output", true);
		
		if(empty($ret)){
			$this->Input->setErrors(array('api' => array('internal' => 'Unable to cancel the service')));
		}
		
		return null;
	}
	
	
	/**
	 * Adds the module row on the remote server. Sets Input errors on failure,
	 * preventing the row from being added.
	 *
	 * @param array $vars An array of module info to add
	 * @return array A numerically indexed array of meta fields for the module row containing:
	 * 	- key The key for this meta field
	 * 	- value The value for this key
	 * 	- encrypted Whether or not this field should be encrypted (default 0, not encrypted)
	 */
	function addModuleRow(array &$vars) {
		$meta_fields = array("server_name", "apiuser", "apikey", "host");
		$encrypted_fields = array("apiuser", "apikey");
		
		$this->Input->setRules($this->getRowRules($vars));
		
		// Validate module row
		if ($this->Input->validates($vars)) {

			// Build the meta data for this row
			$meta = array();
			foreach ($vars as $key => $value) {
				
				if (in_array($key, $meta_fields)) {
					$meta[] = array(
						'key' => $key,
						'value' => $value,
						'encrypted' => (in_array($key, $encrypted_fields) ? 1 : 0)
					);
				}
			}
			
			return $meta;
		}
	}

	/**
	 * Generates a username from the given host name
	 *
	 * @param string $host_name The host name to use to generate the username
	 * @return string The username generated from the given hostname
	 */
	private function generateUsername($host_name) {
		// Remove everything except letters and numbers from the domain
		// ensure no number appears in the beginning
		$username = ltrim(preg_replace('/[^a-z0-9]/i', '', $host_name), '0123456789');

		$length = strlen($username);
		$pool = "abcdefghijklmnopqrstuvwxyz0123456789";
		$pool_size = strlen($pool);
		
		if ($length < 5) {
			for ($i=$length; $i<8; $i++) {
				$username .= substr($pool, mt_rand(0, $pool_size-1), 1);
			}
			$length = strlen($username);
		}
		
		return substr($username, 0, min($length, 8));
	}
	
	/**
	 * Retrieves a list of rules for validating adding/editing a module row
	 *
	 * @param array $vars A list of input vars
	 * @return array A list of rules
	 */
	private function getRowRules(array &$vars) {
		$meta_fields = array("server_name", "apiuser", "apikey", "host");
		$encrypted_fields = array("apiuser", "apikey");
		return array(
			'server_name' => array(
				'empty' => array(
					'rule' => "isEmpty",
					'negate' => true,
					'message' => Language::_("webuzo.!error.server_name.empty", true)
				)
			),
			'apiuser' => array(
				'empty' => array(
					'rule' => "isEmpty",
					'negate' => true,
					'message' => Language::_("webuzo.!error.key.empty", true)
				)
			),
			'apikey' => array(
				'empty' => array(
					'rule' => "isEmpty",
					'negate' => true,
					'message' => Language::_("webuzo.!error.apikey.empty", true)
				)
			),
			'host' => array(
				'format' => array(
					'rule' => "isEmpty",
					'negate' => true,
					'message' => Language::_("webuzo.!error.host.format", true)
				)
			)
		);
	}
	
	/**
	 * Attempts to validate service info. This is the top-level error checking method. Sets Input errors on failure.
	 *
	 * @param stdClass $package A stdClass object representing the selected package
	 * @param array $vars An array of user supplied info to satisfy the request
	 * @return boolean True if the service validates, false otherwise. Sets Input errors when false.
	 */
	function validateService($package, array $vars=null, $edit=false) {
		// Set rules
		$rules = array(
			'webuzo_domain' => array(
				'format' => array(
					'rule' => array(array($this, "validateHostName")),
					'message' => 'Domain Not Valid'
				)
			)
		);
		
		$this->Input->setRules($rules);
		return $this->Input->validates($vars);
	}
	
	/**
	 * Validates that the given hostname is valid
	 *
	 * @param string $domain The host name to validate
	 * @return boolean True if the hostname is valid, false otherwise
	 */
	function validateHostName($domain) {
		$num = explode('.',$domain);
		
		// Domains
		if(preg_match ('/^(?:[a-z0-9][-a-z0-9]*\.\-)*(?:[a-z0-9][-a-z0-9]{0,62})\.(?:(?:[a-z]{2}\.\-)?[a-z0-9\-\.]*)$/i', $domain)){
			return true;
		}
		
		// IPv4
		if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)){
			return true;
		}
		
		// IPv6
		if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
			return true;
		}
		
		return false;	
		
	}
	
	// Error function
	function error($ip = ''){
		
		$err = '';
		
		if(!empty($GLOBALS['webuzo_curl_err'])){
			$err .= ' Curl Error: '.$GLOBALS['webuzo_curl_err'];
		}
		
		if(!empty($ip)){
			$err .= ' (Server IP : '.$ip.')';
		}
		
		return $err;
	}
	
	function wlog($data){
		file_put_contents(__DIR__ .'/logs', $data."\n\n", FILE_APPEND);
	}
	
	function make_api_call($ip, $apiuser = 'root', $pass, $path, $post = array(), $cookies = array()){
		
		global $webuzo_conf;
		
		$apikey = $pass;
		
		$url = 'https://'.$ip.':2005/'.$path;
		$url .= (strstr($url, '?') ? '' : '?');	
		$url .= '&api=serialize&apiuser='.$apiuser.'&apikey='.rawurlencode($apikey).'&skip_callback=1';
	
		if($this->loglevel > 1){
			$this->wlog('[Webuzo:make_api_call] URL : '.$url."\n Post : ".var_export($post, 1));
		}
		
		// Set the curl parameters.
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
			
		// Time OUT
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
		
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			
		// UserAgent
		curl_setopt($ch, CURLOPT_USERAGENT, 'Softaculous');
		
		// Cookies
		if(!empty($cookies)){
			curl_setopt($ch, CURLOPT_COOKIESESSION, true);
			curl_setopt($ch, CURLOPT_COOKIE, http_build_query($cookies, '', '; '));
		}
		
		if(!empty($post)){
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
		}
		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		
		// Get response from the server.
		$resp = curl_exec($ch);
		
		if(empty($resp)){
			$GLOBALS['webuzo_curl_err'] = curl_error($ch);
		}
			
		curl_close($ch);
		
		// The following line is a method to test
		//if(preg_match('/sync/is', $url)) echo $resp;
		
		if(empty($resp)){
			return false;
		}
		
		// As a security prevention measure - Though this cannot happen
		$resp = str_replace($pass, '12345678901234567890123456789012', $resp);
		
		$r = unserialize($resp);
		
		if(empty($r)){
			return false;
		}
		
		return $r;
	}	

	function e_make_api_call($ip, $apiuser = 'root', $pass, $loginAs, $path, $post = array(), $cookies = array()){
		
		global $webuzo_conf;
		
		$apikey = $pass;
		
		$url = 'https://'.$ip.':2003/'.$path;	
		$url .= (strstr($url, '?') ? '' : '?');	
		$url .= '&loginAs='.$loginAs.'&api=serialize&apiuser='.$apiuser.'&apikey='.rawurlencode($apikey).'&skip_callback=1';
	
		if($this->loglevel > 0){
			$this->wlog('[Webuzo] Enduser Call URL : '.$url);
		}
		
		// Set the curl parameters.
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		
		// Time OUT
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
		
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			
		// UserAgent
		curl_setopt($ch, CURLOPT_USERAGENT, 'Softaculous');
		
		// Cookies
		if(!empty($cookies)){
			curl_setopt($ch, CURLOPT_COOKIESESSION, true);
			curl_setopt($ch, CURLOPT_COOKIE, http_build_query($cookies, '', '; '));
		}
		
		if(!empty($post)){
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
		}
		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		
		// Get response from the server.
		$resp = curl_exec($ch);
		curl_close($ch);
		
		// The following line is a method to test
		//if(preg_match('/os/is', $url)) echo $resp;
		
		if(empty($resp)){
			return false;
		}
		
		// As a security prevention measure - Though this cannot happen
		$resp = str_replace($pass, '12345678901234567890123456789012', $resp);
		
		$r = unserialize($resp);
		
		if(empty($r)){
			return false;
		}
		
		return $r;
	}
	
	// Make a call but not an API call
	function enduser_call($ip, $apiuser = 'root', $pass, $loginAs, $path, $post = array(), $cookies = array()){
		
		global $webuzo_conf;
		
		$apikey = $pass;
		
		$url = 'https://'.$ip.':2003/'.$path;	
		$url .= (strstr($url, '?') ? '' : '?');	
		$url .= '&loginAs='.$loginAs.'&apiuser='.$apiuser.'&apikey='.rawurlencode($apikey).'&from_billing=1&skip_callback=1';
	
		if($this->loglevel > 0){
			$this->wlog('[Webuzo] Enduser Call URL : '.$url);
		}
		
		// Set the curl parameters.
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		
		// Time OUT
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
		
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			
		// UserAgent
		curl_setopt($ch, CURLOPT_USERAGENT, 'Softaculous');
		
		// Cookies
		if(!empty($cookies)){
			curl_setopt($ch, CURLOPT_COOKIESESSION, true);
			curl_setopt($ch, CURLOPT_COOKIE, http_build_query($cookies, '', '; '));
		}
		
		if(!empty($post)){
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
		}
		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		
		// Get response from the server.
		$resp = curl_exec($ch);
		curl_close($ch);
		
		// The following line is a method to test
		//if(preg_match('/os/is', $url)) echo $resp;
		
		if(empty($resp)){
			return false;
		}
		
		// As a security prevention measure - Though this cannot happen
		$resp = str_replace($pass, '12345678901234567890123456789012', $resp);
		
		return $resp;
		
	}
	
	// Make a call but not an API call
	function enduser_static_call($ip, $path, $post = array(), $cookies = array()){
		
		global $webuzo_conf;
		
		$apikey = $pass;
		
		$url = 'https://'.$ip.':2003/'.$path;
	
		if($this->loglevel > 1){
			$this->wlog('[Webuzo] Static Call URL : '.$url);
		}
		
		// Set the curl parameters.
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		
		// Time OUT
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
		
		// Turn off the server and peer verification (TrustManager Concept).
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
			
		// UserAgent
		curl_setopt($ch, CURLOPT_USERAGENT, 'Softaculous');
		
		// Cookies
		if(!empty($cookies)){
			curl_setopt($ch, CURLOPT_COOKIESESSION, true);
			curl_setopt($ch, CURLOPT_COOKIE, http_build_query($cookies, '', '; '));
		}
		
		if(!empty($post)){
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
		}
		
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		
		// Get response from the server.
		$resp = curl_exec($ch);
		
		// The following line is a method to test
		//if(preg_match('/os/is', $url)) echo $resp;
		
		if(empty($resp)){
			return false;
		}
		
		header('Cache-Control: public, max-age=600');
		header('Content-Type: '.curl_getinfo($ch, CURLINFO_CONTENT_TYPE));
		header('Content-Length: '.curl_getinfo($ch,  CURLINFO_CONTENT_LENGTH_DOWNLOAD));
		
		echo $resp;
		
		curl_close($ch);
		
	}
	
	function getdecryptpass($enc_pass){
		
		$dec_path = dirname(dirname(dirname(dirname(__FILE__)))).'/app/app_model.php';
		
		include_once($dec_path);
		
		$obj = new AppModel();
		
		return $obj->systemDecrypt($enc_pass);
		
	}
}

